describe("visor.ui-router", function () {
    describe("state change", function () {
        var defer = null;
        angular.module("test.states", ['visor.ui-router', 'ui.router']).config(function ($stateProvider, visorPermissionsProvider) {
            $stateProvider.state("first", {
                url: "/first",
                restrict: function () {
                    return true;
                }
            }).state("deny", {
                url: "/deny",
                restrict: function () {
                    return false;
                }
            });
            visorPermissionsProvider.doBeforeFirstCheck.push(function ($q) {
                defer = $q.defer();
                return defer.promise;
            });

        })
        beforeEach(module("test.states"));

        it("should wait to change state until delay called", inject(function ($location, $state, $rootScope) {
            var successCounter = 0;
            $rootScope.$on("$stateChangeSuccess", function () {
                successCounter++;
            });
            $location.url("/first");
            $rootScope.$apply();
            expect(successCounter).toEqual(0);
            defer.resolve("");
            $rootScope.$apply();
            expect(successCounter).toEqual(1);
            expect($location.url()).toEqual("/first");
            expect($state.current.name).toEqual("first");
        }));

        it("should stop state change when permission rejected", inject(function ($location, $state, $rootScope) {
            $location.url("/deny");
            $rootScope.$apply();
            defer.reject("");
            $rootScope.$apply();
            expect($state.current.name).toEqual("");
        }));
    });
    describe("permissions", function () {
        var calls = [];
        describe('property based inheritance', function () {
            angular.module("test.states.permissions", ['visor.ui-router', 'ui.router']).config(function ($stateProvider, visorPermissionsProvider) {
                $stateProvider.state("parent", {
                    restrict: function () {
                        calls.push("parent");
                        return true;
                    }
                }).state("child", {
                    parent: "parent",
                    url: "/child",
                    restrict: function () {
                        calls.push("child");
                        return true;
                    }
                }).state('grandchild', {
                    url: '/grandchild',
                    parent: "child",
                    restrict: function () {
                        calls.push('grandchild');
                        return true;
                    }
                }).state("deny", {
                    url: "/deny",
                    restrict: function () {
                        calls.push("deny");
                        return false;
                    }
                });

            });
            beforeEach(function () {
                calls = [];
                module("test.states.permissions")
            });
            it("should check permissions in next", inject(function ($state, $rootScope, $location) {
                $state.go("deny");
                $rootScope.$apply();
                expect(calls).toEqual(["deny"])
            }));
            it("should check permission for parent route", inject(function ($state, $rootScope, $location) {
                $state.go("child");
                $rootScope.$apply();
                expect(calls).toEqual(["parent", "child"])
            }));
            it('should check permission for grandparent route', inject(function ($state, $rootScope, $location) {
                $state.go("grandchild");
                $rootScope.$apply();
                expect(calls).toEqual(['parent', 'child', 'grandchild'])
            }));
            it('should check the correct route with getPermissionsForRoute', inject(function ($rootScope, visorPermissions) {
                var permission = visorPermissions.checkPermissionsForRoute('deny');
                expect(permission).toEqual(false);
                permission = visorPermissions.checkPermissionsForRoute('child');
                expect(permission).toEqual(true);
                expect(calls).toEqual(["deny", "parent", "child"])
            }));
            it('should return undefined when route does not exist', inject(function ($rootScope, visorPermissions) {
                var result = visorPermissions.checkPermissionsForRoute('zzz');
                $rootScope.$apply();
                expect(result).toEqual(undefined);
            }));
        });

        describe('dot based inheritance', function () {
            angular.module("test.states.permissions.dot", ['visor.ui-router', 'ui.router']).config(function ($stateProvider, visorPermissionsProvider) {
                $stateProvider.state("parent", {
                    restrict: function () {
                        calls.push("parent");
                        return true;
                    }
                }).state("parent.child", {
                    url: "/child",
                    restrict: function () {
                        calls.push("child");
                        return true;
                    }
                }).state('parent.child.grandchild', {
                    url: '/grandchild',
                    restrict: function () {
                        calls.push('grandchild');
                        return true;
                    }
                });
            });
            beforeEach(function () {
                calls = [];
                module("test.states.permissions.dot")
            });
            it('should check permission for grandparent route', inject(function ($state, $rootScope, $location) {
                $state.go("parent.child.grandchild");
                $rootScope.$apply();
                expect(calls).toEqual(['parent', 'child', 'grandchild'])
            }));
        })
    });
    it("should not change anything if ui-router is not depended on", function () {
        module("visor.permissions", "visor.ui-router")
        inject(function ($location, $rootScope, visorPermissions) {
            $location.url("/something");
            $rootScope.$apply();
            visorPermissions.onRouteChange({
                restrict: function () {
                    return false;
                }
            }, function () {
            });
            $rootScope.$apply();
            //nothing crushed!
        })
    });
});
